/*
 * $Id: display.c,v 1.18 1995/05/27 16:54:47 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <faces.h>
#include <X11/IntrinsicP.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include "mail.h"
#include "../patchlevel.h"

Display        *display;
Window          window;
Screen         *screen;
GC              gc;
XtAppContext    app;
Widget          toplevel;
FacesResources  facesRec;
XtIntervalId    timeOut;


/*
 * local prototypes
 */
#if NeedFunctionPrototypes
void displayText(char *txt, int x, int y, int width);
#else
void displayText();
#endif

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void initDisplay(int *argc, char **argv)
#else
void initDisplay(argc, argv)
  int   *argc;
  char **argv;
#endif
{
  XGCValues      gc_val;
  unsigned long  gc_mask;

  /* Create an application */
  toplevel = XtAppInitialize(&app, "Faces", options, XtNumber(options),
			     argc, argv, NULL, NULL, 0);
  
  /* Get all the resources */
  XtGetApplicationResources(toplevel, &facesRec,
			    resources, XtNumber(resources),
			    NULL, 0);

  /* If asked for version print this and exit. */
  if (facesRec.Version) {
    printf("%s Version %s\n", argv[0], FACES_VERSION);
    exit(0);
  }

  /* Get display and screen */
  display = XtDisplay(toplevel);
  screen  = XtScreen(toplevel);

  /* Send input to the toplevel */
  XtVaSetValues(toplevel, XtNinput, True, NULL);

  /* Resize the toplevel to one image */
  XtResizeWidget(toplevel, facesRec.Maxwidth, facesRec.Maxheight, 0);

  /* Realize the toplevel widget */
  XtRealizeWidget(toplevel);

  /* Now get the window */
  window  = XtWindow(toplevel);

  /* Create an GC */
  gc_mask = GCForeground | GCBackground | GCFunction | GCGraphicsExposures |
            GCFillStyle  | GCFont;
  gc_val.foreground = facesRec.Foreground;
  gc_val.background = facesRec.Background;
  gc_val.function   = GXcopy;
  gc_val.fill_style = FillStippled;
  gc_val.font       = facesRec.Font->fid;
  gc_val.graphics_exposures = False;
  gc  = XCreateGC(display, window, gc_mask, &gc_val);

  /* Select event types wanted */
  XSelectInput(display, window, ExposureMask | ButtonPressMask);
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void setIcon(Face *face, char *name)
#else
void setIcon(face, name)
  Face *face;
  char *name;
#endif
{
  if (face != NULL) {
    /*
     * Display an icon.
     */
    if (face->icon != None)
      XtVaSetValues(toplevel, XtNiconPixmap, face->icon, NULL);
    else if (face->image != None)
      XtVaSetValues(toplevel, XtNiconPixmap, face->image, NULL);

    /*
     * Change the icon name.
     */
    if (name == NULL)
      XtVaSetValues(toplevel, XtNiconName, face->name, NULL);
    else
      XtVaSetValues(toplevel, XtNiconName, name, NULL);

  } else
    XtVaSetValues(toplevel, XtNiconName, name, NULL);
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void displayFaces(void)
#else
void displayFaces()
#endif
{
  int           images_x, images_y, x, y, w, h;
  Boolean       compact, last;
  unsigned int  width, height;
  Face         *face;
  Mail         *mail;

  images_x     = 0;
  images_y     = 0;
  mail         = NULL;
  last         = False;
  compact      = facesRec.CompactImages;

  /*
   * If firstMail is NULL it means there is no mail, and then just display
   * the nomail face.
   */
  if (firstMail == NULL) {
    face    = nomailFace;
    compact = True;
  } else {
    mail = firstMail;
    face = firstFace;
  }

  /*
   * Display all the images
   */
  while (!last) {
    if (!compact)
      face = mail->face;

    if (face->counter != 0)
      XCopyArea(display, face->image, window, gc,
		0, 0, facesRec.Maxwidth, facesRec.Maxheight,
		images_x*facesRec.Maxwidth, images_y*facesRec.Maxheight);
    
    /*
     * Draw the name
     */
    if (facesRec.FromX < 0)
      x = (images_x+1)*facesRec.Maxwidth + facesRec.FromX;
    else
      x = images_x*facesRec.Maxwidth + facesRec.FromX;
    
    if (facesRec.FromY < 0)
      y = (images_y+1)*facesRec.Maxheight + facesRec.FromY;
    else {
      h = facesRec.Font->max_bounds.ascent - facesRec.Font->max_bounds.descent;
      y = images_y*facesRec.Maxheight + facesRec.FromY + h;
    }

    if (facesRec.FromX < 0)
      w = -facesRec.FromX;
    else
      w = facesRec.Maxwidth - facesRec.FromX;

    if (!compact)
      displayText(mail->from, x, y, w);

    if (compact && (strncmp(face->name, "unknown", 7) != 0))
      displayText(face->name, x, y, w);
      
    /*
     * Go to the next image.
     */
    if (compact) {
      face = face->next;
      if (face == NULL)
	last = True;
    } else {
      mail = mail->next;
      if (mail == NULL)
	last = True;
    }

    /*
     * Increase the image counter. If there are more than MaxImages images
     * in a row than go the next row.
     */
    images_x++;
    if (images_x >= facesRec.MaxImages) {
      images_y++;
      images_x=0;
    }
  }

  /*
   * Clean up the rest of the window
   */
  if (images_y > 0) {
    XSetForeground(display, gc, facesRec.Background);
    XFillRectangle(display, window, gc,
		   images_x*facesRec.Maxwidth, images_y*facesRec.Maxheight,
		   (facesRec.MaxImages - images_x)*facesRec.Maxwidth,
		   facesRec.Maxheight);
    XSetForeground(display, gc, facesRec.Foreground);
  }

  /*
   * Create an icon. This will always show the face and the name of the
   * person from who you received the last mail.
   */
  if (firstMail == NULL)
    setIcon(nomailFace, "No Mail");
  else {
    mail = firstMail;
    while ((mail->next != NULL) && !facesRec.ShowFirst)
      mail = mail->next;
    setIcon(mail->face, mail->from);
  }

  /*
   * Calculate the new window width and height.
   */
  if (images_x == 0) {
    images_y--;
    images_x = facesRec.MaxImages;
  }
  height = facesRec.Maxheight * (images_y+1);

  if (images_y > 0)
    width  = facesRec.Maxwidth * facesRec.MaxImages;
  else
    width  = facesRec.Maxwidth * images_x;

  /*
   * Resize the window so all images are shown.
   */
  XtResizeWidget(toplevel, width, height, 0);

  /*
   * Without a flush X won't display.
   */
  XFlush(display);
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void displayText(char *txt, int x, int y, int width)
#else
void displayText(txt, x, y, width)
  char *txt;
  int   x, y, width;
#endif
{
  int          len, dummy;
  XCharStruct overall;

  len = strlen(txt);

  /*
   * Get the max number of characters that can be displayed.
   */
  while (len > 0) {
    XTextExtents(facesRec.Font, txt, len, &dummy, &dummy, &dummy, &overall);
    if (overall.width < width)
      break;
    len--;
  }

  if (len == 0)
    return;
 
  /*
   * Draw the text at the requested place.
   */
  XSetFunction(display, gc, GXxor);
  XDrawImageString(display, window, gc, x, y, txt, len);
  XSetFunction(display, gc, GXcopy);      
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void alarmHandler(XtPointer userData, XtIntervalId *id)
#else
void alarmHandler(userData, id)
  XtPointer     userData;
  XtIntervalId *id;
#endif
{
  int newFaces;

  /*
   * Read the mailbox and create the images
   */
  newFaces = parseMail();

  /*
   * Regenerate the window
   */
  if (newFaces != 0) {
    if (facesRec.Debug)
      printf("Generated %d new faces.\n", newFaces);
    if (newFaces > 0)
      XRaiseWindow(display, window);
    displayFaces();
  }

  /*
   * Restart the timer for another look at the mailbox.
   */
  timeOut = XtAppAddTimeOut(app, facesRec.CheckMail*1000, alarmHandler, NULL);
}

