/*
 * $Id: main.c,v 1.28 1995/05/27 16:55:38 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <faces.h>
#include "mail.h"

Mail       *firstMail;

/*
 * local prototypes
 */
#if NeedFunctionPrototypes
void shutdown (void);
void clearData(void);
void usage    (char *);
#else
void shutdown ();
void clearData();
void usage    ();
#endif

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void main(int argc, char **argv)
#else
void main(argc, argv)
  int    argc;
  char **argv;
#endif
{
  XEvent  report;
  pid_t   child;
  char   *env, *newMail;

  /*
   * Initialize some global variables.
   */
  firstMail   = NULL;

  /*
   * Initialize the faces library.
   */
  FaceInit();

  /*
   * Open a window on the display, and create a GC.
   */
  initDisplay(&argc, argv);

  /*
   * No more commandline options should be left, otherwise print help.
   */
  if (argc > 1)
    usage(argv[0]);

  /*
   * If no mailfile was given then check MAIL, otherwise quit.
   */
  if (facesRec.MailFile == NULL) {
    env = getenv("MAIL");
    if (env == NULL) {
      fprintf(stderr, "Could not locate mailfolder.\n");
      exit(-1);
    }
    facesRec.MailFile = XtNewString(env);
  }

  /*
   * Replace ~ with $HOME
   */
  if (facesRec.MailFile[0] == '~') {
    env = getenv("HOME");
    if (env == NULL) {
      fprintf(stderr, "Could not locate the variable HOME.\n");
      exit(-1);
    }
    newMail = XtMalloc(strlen(facesRec.MailFile) + strlen(env));
    strcpy(newMail, env);
    strcat(newMail, (facesRec.MailFile+1));
    facesRec.MailFile = XtNewString(newMail);
    XtFree(newMail);
  }
    

  /*
   * Change some options of the faces library.
   */
  facesOptions.Background     = facesRec.Background; 
  facesOptions.DatabaseOrder  = XtNewString(facesRec.DatabaseOrder);
  facesOptions.Debug          = facesRec.Debug;
  facesOptions.ImageOrder     = XtNewString(facesRec.LoadOrder);
  facesOptions.SearchFunction = facesRec.Algorithm;
  facesOptions.PathNames      = XtNewString(facesRec.PathNames);
  facesOptions.TmpFile        = XtNewString(facesRec.TmpFace);
  facesOptions.UseColorIcon   = facesRec.UseColorIcon;
  facesOptions.UseMailcap     = facesRec.UseMailcap;
  facesOptions.XfaceFirst     = facesRec.XfaceFirst;

  /*
   * Go in the background.
   */
  if (!facesRec.Debug) {
    child = fork();
    if (child < 0)
      fprintf(stderr, "Could not fork in the background.\n");
    if (child > 0)
      exit(0);
  }

  /*
   * Initialize the database structure
   */
  FaceInitDB();

  /*
   * Load the face for nomail.
   */
  nomailFace = loadNomail();

  /*
   * Start the alarm, make it a short period so the mail gets read.
   */
  timeOut = XtAppAddTimeOut(app, 1, alarmHandler, NULL);

  /*
   * Wait for an event to happen and do the right thing. 
   */
  while (1) {
    XtAppNextEvent(app, &report);
    switch (report.type) {
    case Expose:                           /* Redraw the screen     */
      displayFaces();
      break;
    case ButtonPressMask:                  /* Close down gracefully */
      switch(report.xbutton.button) {
      case Button1:
	XtRemoveTimeOut(timeOut);
	clearData();
	alarmHandler(NULL, &timeOut);
	break;
      case Button3:
	shutdown();
	break;
      }
      break;
    default:
      break;
    }
  }
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void clearData(void)
#else
void clearData()
#endif
{
  Mail    *mail;

  /*
   * Remove all the Mail.
   */
  while (firstMail != NULL) {
    mail = firstMail->next;
    XtFree(firstMail->from);
    XtFree(firstMail->subject);
    XtFree((char *)firstMail);
    firstMail = mail;
  }

  /*
   * Remove all the faces.
   */
  FaceRemoveAll();
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void shutdown(void)
#else
void shutdown()
#endif
{
  /*
   * Remove all the regular data
   */
  clearData();

  /*
   * Remove all the databases.
   */
  FaceCloseDB();

  /*
   * Shutdown the display.
   */
  XtDestroyWidget(toplevel);

  /*
   * And exit the program.
   */
  exit(0);
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void usage(char *progname)
#else
void usage(progname)
  char *progname;
#endif
{
  char *fmt;

  fmt = XtNewString("Usage: %s [-options] [-xoptions]\n\
       -m <mailbox> : Mailbox to display.\n\
       -d           : Give debug information.\n\
       -v           : Show version and exit.\n\n\
  Besides these options the regular X options are recognized.\n");

  fprintf(stderr, fmt, progname);

  XtFree(fmt);
  exit(1);
}
