/*
 * $Id: misc.c,v 1.11 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include "facesP.h"

/*
 * The two structures used in the library, a private and a public one.
 */
FacesPrivateStruct FacesPrivate;                /* The private structure */
FacesOptions       facesOptions;                /* ThUnknowne public structure  */

/*
 * The global variables that are made public.
 */
Face              *firstFace;

/*************************************************************************
 * Initialization of the variables
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceInit(void)
#else
void FaceInit()
#endif
{
  facesOptions.Background     = -1;
  facesOptions.DatabaseOrder  = NULL;
  facesOptions.Debug          = 0;
  facesOptions.Foreground     = -1;
  facesOptions.ImageOrder     = NULL;
  facesOptions.PathNames      = NULL;
  facesOptions.SearchFunction = searchKnown;
  facesOptions.TmpFile        = NULL;
  facesOptions.UseMailcap     = True;
#if defined(sgi)
  facesOptions.UseColorIcon   = True;
#else
  facesOptions.UseColorIcon   = False;
#endif
  facesOptions.XfaceFirst     = False;

  FacesPrivate.databaseFlags  = 0;
  FacesPrivate.depth          = 0;
  FacesPrivate.firstDB        = NULL;
  FacesPrivate.gc             = NULL;
  FacesPrivate.mailcap        = NULL;
  
  firstFace                   = NULL;
}

/*************************************************************************
 * Add the face to the list of faces already loaded
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceClose(void)
#else
void FaceClose()
#endif
{
  FacesDB *db;

  /*
   * Remove all the database stuff.
   */
  while (FacesPrivate.firstDB != NULL) {
    db = FacesPrivate.firstDB->next;
    XtFree(FacesPrivate.firstDB->path);
    XtFree((char *)FacesPrivate.firstDB);
    FacesPrivate.firstDB = db;
  }

  /*
   * Remove all the faces
   */
  FaceRemoveAll();
}

/*************************************************************************
 * Remove a single face from the list.
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceRemoveOne(Face *face)
#else
void FaceRemoveOne(face)
  Face *face;
#endif
{
  Face *prev, *index;

  /*
   * Find the face in the list
   */
  prev  = NULL;
  index = firstFace;
  while ((index != NULL) && (strcmp(index->name, face->name) != 0)) {
    prev  = index;
    index = index->next;
  }

  /*
   * If the face was in the list, then remove if from the list.
   */
  if (index != NULL) {
    if (prev == NULL)
      firstFace  = index->next;
    else
      prev->next = index->next;
    if (index->next != NULL)
      index->next->prev = prev;
  }

  /*
   * Remove the face.
   */
  XtFree(face->name);
  XtFree((char *)face);
}

/*************************************************************************
 * Remove all the faces from the list.
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceRemoveAll(void)
#else
void FaceRemoveAll()
#endif
{
  while (firstFace != NULL)
    FaceRemoveOne(firstFace);
}

/*************************************************************************
 * Decrease the counter and remove if not in use.
 *************************************************************************/
#if NeedFunctionPrototypes
void FaceDelete(Face *face)
#else
void FaceDelete(face)
  Face *face;
#endif
{
  /*
   * Safety.
   */
  if (face == NULL)
    return;

  /*
   * Decrease the counter.
   */
  if (face->counter > 0)
    face->counter--;

  /*
   * Check if the icon is in another face.
   */
  if (face->iconPtr != NULL)
    FaceDelete(face->iconPtr);

  /*
   * Delete the face only when the counter is 0.
   */
  if (face->counter == 0) {
    if (facesOptions.Debug)
      printf("Removing face of %s\n", face->name);
    FaceRemoveOne(face);
  }
}

/*************************************************************************
 * Add the face to the list of faces already loaded
 *************************************************************************/
#if NeedFunctionPrototypes
void face_addFace(Face *newface)
#else
void face_addFace(newface)
  Face *newface;
#endif
{
  Face *index;

  if ((newface == NULL) || (newface->name == NULL))
    return;
  
  if (firstFace == NULL) {
    firstFace = newface;
    return;
  }

  index = firstFace;
  while ((index->next != NULL) && (strcmp(newface->name, index->name) != 0))
    index = index->next;

  if (strcmp(newface->name, index->name) != 0) {
    index->next = newface;
    newface->prev = index;
  }
}

/*************************************************************************
 * Do the real initialization
 *************************************************************************/
#if NeedFunctionPrototypes
void face_init(void)
#else
void face_init()
#endif
{
  char          *env, *orig;
  XGCValues      gc_val;
  unsigned long  gc_mask;

  /*
   * Initilize the mailcap
   */
  face_readMailcap();

  /*
   * Check for some environment variables.
   */
  env = getenv("FACEPATH");
  if (env != NULL) {
    if (facesOptions.PathNames != NULL) {
      orig = XtNewString(facesOptions.PathNames);
      facesOptions.PathNames = XtMalloc(strlen(env) + strlen(orig) + 2);
      strcpy(facesOptions.PathNames, env);
      strcat(facesOptions.PathNames, ":");
      strcat(facesOptions.PathNames, orig);
      XtFree(orig);
    } else
      facesOptions.PathNames = XtNewString(env);
  }
#ifdef FACEPATH
  orig = facesOptions.PathNames;
  if (orig == NULL)
    orig = XtNewString(FACEPATH);
  else if (orig[strlen(orig)] == ':') {
    orig = XtRealloc(orig, strlen(orig) + strlen(FACEPATH));
    strcat(orig, FACEPATH);
  }
  facesOptions.PathNames = orig;
#endif

  env = getenv("FACEDB");
  if (env != NULL) {
    if (facesOptions.DatabaseOrder != NULL) {
      orig = XtNewString(facesOptions.DatabaseOrder);
      facesOptions.DatabaseOrder = XtMalloc(strlen(env) + strlen(orig) + 1);
      strcpy(facesOptions.DatabaseOrder, env);
      strcat(facesOptions.DatabaseOrder, ":");
      strcat(facesOptions.DatabaseOrder, orig);
      XtFree(orig);    
    } else
      facesOptions.DatabaseOrder = XtNewString(env);
  }
#ifdef FACEDB
  orig = facesOptions.DatabaseOrder;
  if (orig == NULL)
    orig = XtNewString(FACEDB);
  else if (orig[strlen(orig)] == ':') {
    orig = XtRealloc(orig, strlen(orig) + strlen(FACEDB));
    strcat(orig, FACEDB);
  }
  facesOptions.DatabaseOrder = orig;
#endif

  /*
   * Get the fore and background colors.
   */
  if (facesOptions.Background == -1)
    facesOptions.Background = WhitePixelOfScreen(screen);
  if (facesOptions.Foreground == -1)
    facesOptions.Foreground = BlackPixelOfScreen(screen);

  /*
   * Get the depth of the screen.
   */
  FacesPrivate.depth = DefaultDepthOfScreen(screen);

  /*
   * Create a GC
   */
  gc_mask = GCForeground | GCBackground | GCFunction |
            GCGraphicsExposures | GCFillStyle;
  gc_val.foreground = facesOptions.Foreground;
  gc_val.background = facesOptions.Background;
  gc_val.function   = GXcopy;
  gc_val.fill_style = FillStippled;
  gc_val.graphics_exposures = False;
  FacesPrivate.gc  = XCreateGC(display, window, gc_mask, &gc_val);

  /*
   * Set the imageOrder.
   */
  if (facesOptions.ImageOrder == NULL)
#ifdef XPM
    facesOptions.ImageOrder = XtNewString("gif:xpm:xbm");
#else
    facesOptions.ImageOrder = XtNewString("xbm");
#endif

  /*
   * Set the TmpFile.
   */
  if (facesOptions.TmpFile == NULL)
    facesOptions.TmpFile = XtNewString(TMPFILE);
}
